dofile_once("mods/random_start_events/utilities.lua")

--This might be a little confusing, but 100 probability doesn't mean that it will always be chosen, it just means that it will most likely be chosen over all other events
--This is because if there are multiple events with the same probability, it will choose one of them at random
--0 probability means that it will never be chosen

events_list = { --Only one event can be chosen per run
    { -- Pee
        message = "You are disgusting", -- The message that will display at the start if this event is chosen. Use $return as a placeholder for the value that will be returned.
        description = "Didn't you have anywhere else to go?", -- Optional, can use $return as   well
        probability = 1, -- Probability of this event being chosen (percentage)
        event = function(player_entity) --These arguments cannot be changed, but put whatever code you like in here
            local x, y = EntityGetTransform(player_entity)
            GameCreateParticle("urine", x, y, 100, 0, 0, false)
        end
    },
    { -- Chaos die
        message = "Roll your fate",
        description = "",
        probability = 10,
        event = function(player_entity)
            local x, y = EntityGetTransform(player_entity)
            EntityLoad("data/entities/items/pickup/" .. RandomItem("physics_die", "physics_greed_die") .. ".xml", x, y)
        end
    },
    { -- Random shift
        message = "Something is not right",
        description = "You hear \"$return\" echoing in your head",
        probability = 20,
        event = function(player_entity)
            local mat =  GetRandomMaterialInclude(false, false, true, true, true, true)
            local mat2 = GetRandomMaterialInclude(false, false, true, true, true, true)
            local from, to = ConvertSameUINameMaterials(mat, mat2, true, false)
            return string.lower(GameTextGetTranslatedOrNot(RandomItem(from,to)))
        end
    },
    { -- Random shift n times
        message = "Something is terribly wrong",
        description = "Multiple materials have shifted",
        probability = 10,
        event = function(player_entity)
            local n = Random(2, 15)

            for i = 1, n do
                ConvertRandomMaterialsAll(false)
            end
        end
    },
    { -- More inventory
        message = "Big pockets",
        description = "Bigger inventory",
        probability = 20,
        event = function(player_entity)
            SetEntityComponentValue(player_entity, "Inventory2Component", "full_inventory_slots_y", Random(2, 3))
        end
    },
    { -- Water to gold
        message = "Scrooge McDuck",
        description = "Swim in gold!",
        probability = 15,
        event = function(player_entity)
            ConvertMaterials("water", "gold")
        end
    },
    { -- Damage to ground on land
        message = "Heavy weight",
        description = "You feel heavier",
        probability = 10,
        event = function(player_entity)
            SetEntityComponentValue(player_entity, "CharacterDataComponent", "destroy_ground", 20)
            SetEntityComponentValue(player_entity, "VelocityComponent", "mass", 10)
        end
    },
    { -- Fire to water
        message = "Waterworks",
        description = "Don't play with fire",
        probability = 2,
        event = function(player_entity)
            ConvertMaterials("fire", "water")
            ConvertMaterials("flame", "water")
            ConvertMaterials("liquid_fire", "water")
        end
    },
    { -- Random pickup item
        message = "The gods bring you a gift",
        description = "Don't be rude",
        probability = 25,
        event = function(player_entity)
            local x, y = EntityGetTransform(player_entity)
            EntityLoad("data/entities/items/pickup/" .. RandomItemFromTable(all_pickups), x + 50, y + 50)
        end
    },
    { -- Rainbow farts
        message = "Rainbow what?",
        description = "Rainbow farts...",
        probability = 2,
        event = function(player_entity)
            AddEffectTo(player_entity, "effect_rainbow_farts.xml", 600)
        end
    },
    { -- Ingest random material
        message = "What did you eat?",
        description = "Something good I hope",
        probability = 20,
        event = function(player_entity)
            local amount = Random(100, 600)
            EntityIngestRandomMaterial(player_entity, amount)
        end
    },
    { -- Give random potion
        message = "Basic Alchemy",
        description = "You get a random potion",
        probability = 15,
        event = function(player_entity)
            local potion = EntityLoad("data/entities/items/pickup/potion.xml", 0,0)
            GamePickUpInventoryItem(player_entity, potion)
        end
    },
    { -- Give random potion with ANY material
        message = "Advanced Chemistry",
        description = "You get a random potion with any material in it",
        probability = 20,
        event = function(player_entity)
            local value = GetRandomMaterialInclude(false, false, true, true, true, true)

            local potion = EntityLoad("mods/random_start_events/files/potion.xml", 0,0)
            AddMaterialInventoryMaterial(potion, value, 1000)
            GamePickUpInventoryItem(player_entity, potion)

        end
    },
    { -- Random material to gold
        message = "Don't get greedy",
        description = "Golden \"$return\"",
        probability = 10,
        event = function(player_entity)
            local value = GetRandomMaterialInclude(true, false, true, true, true, true)

            local mat1, mat2 = ConvertSameUINameMaterials(value, "gold")
            return GameTextGetTranslatedOrNot(mat1)
        end
    },
    { -- Bleed random material
        message = "It's in your blood",
        description = "You feel \"$return\" coursing through your veins",
        probability = 20,
        event = function(player_entity)
            local material = GetRandomMaterialInclude(false, false, true, true, true, false) --Liquids, gases and sands
            SetEntityBlood(player_entity, material)
            return string.lower(GameTextGetTranslatedOrNot(GetMaterialUIKey(material)))
        end
    },
    { -- Random spell
        message = "You are a wizard!",
        description = "You get a random spell",
        probability = 20,
        event = function(player_entity)
            local x,y = EntityGetTransform(player_entity)
            local id = GetRandomAction(x,y, RandomItem(0,0,0,1,1,1,2,2,3,3,4,4,5,6))
            CreateItemActionEntity(id, x, y)
        end
    },
    { -- Total chaos
        message = "Chaos chaos chaos",
        description = "Nothing is the same",
        probability = 3,
        event = function(player_entity)
            mats = table.clone(GetAllMaterialsInclude(true, false, true, true, true, true))
            for i = 1, #mats do
                if(#mats == 0) then return end
                local rand1 = Random(1, #mats)
                local mat1 = mats[rand1]
                table.remove(mats, rand1)

                local rand2 = Random(1, #mats)
                local mat2 = mats[rand2]
                table.remove(mats, rand2)

                if(mat1 and mat2) then
                    ConvertMaterials(mat1, mat2) 
                else
                    print("Error in random material conversion "..i..": "..tostring(mat1)..", "..tostring(mat2))
                end
                
            end
        end
    },
    
}

test_event =  { -- Will always be chosen if probability is more than 0
message = "Test event",
description = "Test \"$return\"",
probability = 0,
event = function(player_entity)
    return "event"
end
}
probability_multiplier = 1 -- The probability of an event being chosen is multiplied by this value
always_choose_event = false