all_pickups = {
    "beamstone.xml", "bloodmoney_10.xml", "bloodmoney_1000.xml", "bloodmoney_10000.xml", "bloodmoney_200.xml", "bloodmoney_200000.xml", "bloodmoney_50.xml", "brimstone.xml", "broken_wand.xml", "cape.xml", "chest_leggy.xml", "chest_random.xml", "chest_random_super.xml", "egg_fire.xml", "egg_hollow.xml", "egg_monster.xml", "egg_purple.xml", "egg_red.xml", "egg_slime.xml", "egg_spiders.xml", "egg_worm.xml", "essence_air.xml", "essence_alcohol.xml", "essence_fire.xml", "essence_laser.xml", "essence_water.xml", "evil_eye.xml", "goldnugget.xml", "goldnugget_10.xml", "goldnugget_1000.xml", "goldnugget_10000.xml", "goldnugget_200.xml", "goldnugget_200000.xml", "goldnugget_50.xml", "gourd.xml", "greed_curse.xml", "heart.xml", "heart_better.xml", "heart_evil.xml", "heart_fullhp.xml", "heart_fullhp_temple.xml", "jar.xml", "jar_of_urine.xml", "moon.xml", "musicstone.xml", "perk.xml", "perk_reroll.xml", "physics_die.xml", "physics_gold_orb.xml", "physics_gold_orb_greed.xml", "physics_greed_die.xml", "poopstone.xml", "potion.xml", "potion_aggressive.xml", "potion_alcohol.xml", "potion_empty.xml", "potion_porridge.xml", "potion_random_material.xml", "potion_secret.xml", "potion_slime.xml", "potion_starting.xml", "potion_vomit.xml", "potion_water.xml", "powder_stash.xml", "random_card.xml", "safe_haven.xml", "spell_refresh.xml", "stonestone.xml", "summon_portal_broken.xml", "thunderstone.xml", "wandstone.xml", "waterstone.xml",
}


function ConvertMaterials(from_material, to_material) -- Converts materials and returns the original material keys for getting the ui name 
    local from_ui = GetMaterialUIKey(from_material)
    local to_ui = GetMaterialUIKey(to_material)

    ConvertMaterialEverywhere(CellFactory_GetType(from_material), CellFactory_GetType(to_material))
    print(from_material.." -> "..to_material)
    return from_ui, to_ui
end

function SetMaxHp(player_entity, max_hp) -- Will divide the hp by 25 since that's what the game uses
    SetEntityComponentValue(player_entity, "DamageModelComponent", "max_hp", max_hp/25)
end

function SetHp(player_entity, hp) -- Will divide the hp by 25 since that's what the game uses
    SetEntityComponentValue(player_entity, "DamageModelComponent", "hp", hp/25)
end

all_materials = nil
function GetAllMaterials(include_statics) --Gets all materials (except fx)

    if(all_materials) then return all_materials -- If the materials are already loaded, return them
    else all_materials = {} end

    local solids = CellFactory_GetAllSolids(include_statics, false)
    local liquids = CellFactory_GetAllLiquids(include_statics, false)
    local gases = CellFactory_GetAllGases(include_statics, false)
    local sands = CellFactory_GetAllSands(include_statics, false)
    local fires = CellFactory_GetAllFires(include_statics, false)

    TableConcat(all_materials, solids)
    TableConcat(all_materials, liquids)
    TableConcat(all_materials, gases)
    TableConcat(all_materials, sands)
    TableConcat(all_materials, fires)
    return all_materials
end

function GetAllMaterialsInclude(include_statics, include_solids, include_liquids, include_gases, include_sands, include_fires) --Gets all materials only including the ones you want
    local materials = {}

    if(include_solids) then TableConcat(materials, CellFactory_GetAllSolids(include_statics, false)) end
    if(include_liquids) then TableConcat(materials, CellFactory_GetAllLiquids(include_statics, false)) end
    if(include_gases) then TableConcat(materials, CellFactory_GetAllGases(include_statics, false)) end
    if(include_sands) then TableConcat(materials, CellFactory_GetAllSands(include_statics, false)) end
    if(include_fires) then TableConcat(materials, CellFactory_GetAllFires(include_statics, false)) end

    return materials
end

function GetRandomMaterialAll(include_statics) --Gets a random material from all materials
    return RandomItemFromTable(GetAllMaterials(include_statics))
end

function GetAllMaterialsType(include_statics, material_type) --Valid types: "solids", "liquids", "gases", "sands", "fires"
    local materials = {}

    material_type = string.lower(material_type)
    if(material_type == "solids") then materials = CellFactory_GetAllSolids(include_statics, false)
    elseif(material_type == "liquids") then materials = CellFactory_GetAllLiquids(include_statics, false)
    elseif(material_type == "gases") then materials = CellFactory_GetAllGases(include_statics, false)
    elseif(material_type == "sands") then materials = CellFactory_GetAllSands(include_statics, false)
    elseif(material_type == "fires") then materials = CellFactory_GetAllFires(include_statics, false) 
    else print("Invalid material type: "..material_type) end
    return materials
end

function GetRandomMaterial(material_type, include_statics) --Valid types: "solid", "liquid", "gas", "sand", "fire"
    local materials = {}

    if(material_type == "solid") then materials = CellFactory_GetAllSolids(include_statics, false)
    elseif(material_type == "liquid") then materials = CellFactory_GetAllLiquids(include_statics, false)
    elseif(material_type == "gas") then materials = CellFactory_GetAllGases(include_statics, false)
    elseif(material_type == "sand") then materials = CellFactory_GetAllSands(include_statics, false)
    elseif(material_type == "fire") then materials = CellFactory_GetAllFires(include_statics, false) end

    return RandomItemFromTable(materials)
end

function GetRandomMaterialInclude(include_statics, include_solids, include_liquids, include_gases, include_sands, include_fires) --Gets a random material only from the materials you want
    return RandomItemFromTable(GetAllMaterialsInclude(include_statics, include_solids, include_liquids, include_gases, include_sands, include_fires))
end

function EntityIngestRandomMaterial(entity_id, amount) --Only ingests solids or liquids (nothing else is ingestable)
    EntityIngestMaterial(entity_id, CellFactory_GetType(GetRandomMaterialInclude(false, true, true)), amount)
end

function RandomItem(...) -- Returns a random item from the arguments, can be anything
    local args = {...}
    return args[Random(1, #args)]
end

function AddEffectTo(entity_id, effect_name, duration) --Adds a potion effect to an entity (without including "data/entities/misc/"in the name)
    local effect_id = LoadGameEffectEntityTo(entity_id, "data/entities/misc/"..effect_name )
    local gec_id = EntityGetFirstComponentIncludingDisabled( effect_id, "GameEffectComponent" )
    ComponentSetValue2( gec_id, "frames", duration * 60)
end

function RandomItemFromTable(table) --Pretty self explanatory
    return table[Random(1, #table)]
end

function ConvertRandomMaterialsAll(include_statics) --All except solids, since those cause a lot of problems
    local materials = GetAllMaterialsInclude(include_statics, false, true, true, true)
    local materials_copy = table.clone(materials)

    local mat1_index = Random(1, #materials_copy)
    local mat1 = materials_copy[mat1_index]
    table.remove(materials_copy, mat1_index)

    local mat2_index = Random(1, #materials_copy)
    if(materials_copy[mat2_index] == "creepy_liquid") then mat2_index = Random(1, #materials_copy) end --If the material to convert to is creepy liquid, pick another one since it causes a lot of problems
    local mat2 = materials_copy[mat2_index]

    return ConvertMaterials(mat1, mat2)
end

function ConvertTypeOfMaterialRandom(from_type_mat, to_type_mat) --Valid types: "solid", "liquid", "gas", "sand", "fire"
    local materials = {}
    local mat1_index = 0
    local mat2_index = 0
    
    local mat1 = nil
    local mat2 = nil
    for i = 1, 2 do
        if(i == 1) then --I whish i could reuse some of the GetRandomMaterial functions, but I need the index, and I'm too lazy to change what is returned by those functions
            if(from_type_mat == "solid") then materials = CellFactory_GetAllSolids(true, false)
            elseif(from_type_mat == "liquid") then materials = CellFactory_GetAllLiquids(true, false)
            elseif(from_type_mat == "gas") then materials = CellFactory_GetAllGases(true, false)
            elseif(from_type_mat == "sand") then materials = CellFactory_GetAllSands(true, false)
            elseif(from_type_mat == "fire") then materials = CellFactory_GetAllFires(true, false) end
            mat1_index = Random(1, #materials)
            mat1 = materials[mat1_index]
            table.remove(materials, mat1_index)
        else
            if(from_type_mat ~= to_type_mat) then --If they're the same there's no need to get the materials twice
                if(to_type_mat == "solid") then materials = CellFactory_GetAllSolids(true, false)
                elseif(to_type_mat == "liquid") then materials = CellFactory_GetAllLiquids(true, false)
                elseif(to_type_mat == "gas") then materials = CellFactory_GetAllGases(true, false)
                elseif(to_type_mat == "sand") then materials = CellFactory_GetAllSands(true, false)
                elseif(to_type_mat == "fire") then materials = CellFactory_GetAllFires(true, false) end
            end
            mat2_index = Random(1, #materials)
            if(materials[mat2_index] == "creepy_liquid") then mat2_index = Random(1, #materials) end --If the material to convert to is creepy liquid, pick another one since it causes a lot of problems
            mat2 = materials[mat2_index]
        end
    end

    return ConvertMaterials(mat1, mat2)
end

function ConvertGroupOfMaterials(from_group, to_material) -- Returns a table of all the converted materials (in material key format)
    local converted = {}
    local to_material_key = GetMaterialUIKey(to_material)
    for i = 1, #from_group do
        local material = from_group[i]
        local mat1, mat2 = ConvertMaterials(material, to_material)
        table.insert(converted, mat1)
    end
    return converted, to_material_key
end

function ConvertSameUINameMaterials(from_material, to_material, include_statics, include_solids, include_liquids, include_gases, include_sands, include_fires) --Returns a table of all the converted materials (in material key format). (Includes all materials by default)
    local ui_name = GameTextGetTranslatedOrNot(GetMaterialUIKey(from_material))
    if(ui_name == "") then ui_name = from_material end

    local grouped = GetAllMaterialsWithSameUIName(ui_name, include_statics, include_solids, include_liquids, include_gases, include_sands, include_fires)
    local from_group, to_material = ConvertGroupOfMaterials(grouped, to_material)
    return ui_name, to_material
end

function GetAllMaterialsWithSameUIName(ui_name, include_statics, include_solids, include_liquids, include_gases, include_sands, include_fires) --Returns a table of all the materials with the same UI name. (Includes all materials by default)
    include_statics = include_statics or true
    include_solids = include_solids or true
    include_liquids = include_liquids or true
    include_gases = include_gases or true
    include_sands = include_sands or true
    include_fires = include_fires or true

    local materials = {}
    for _, material in ipairs(GetAllMaterialsInclude(include_statics, include_solids, include_liquids, include_gases, include_sands, include_fires)) do
        if(GameTextGetTranslatedOrNot(GetMaterialUIKey(material)) == ui_name) then
            table.insert(materials, material)
        end
    end
    return materials
end

function GetMaterialUIKey(material_name) --Returns a material ui key from a material name (use the GameTextGetTranslatedOrNot(material_key) to get the real ui name)
    local material_id = CellFactory_GetType(material_name)
    if(material_id <= 0) then return "" end
    local ui_name = CellFactory_GetUIName(material_id)
    return ui_name
end

function SetEntityMoney(entity_id, value) -- yep.
    SetEntityComponentValue(entity_id, "WalletComponent", "money", value)
end

function GiveEntityItem(entity_id, item_path) -- item_path starting from data/entities/items/
    local x,y = EntityGetTransform(entity_id)
    local id = EntityLoad("data/entities/items/"..item_path, x, y)
    GamePickUpInventoryItem(entity_id, id)
end

function GiveEntitySpell(entity_id, spell_path)-- spell_path starting from data/entities/projectiles/deck/
    local x,y = EntityGetTransform(entity_id)
    local id = EntityLoad("data/entities/projectiles/deck/"..spell_path, x, y)
    GamePickUpInventoryItem(entity_id, id)
end 

function SetEntityComponentValue(entity_id, component, parameter, value) --Sets the value of a component in the entity
    local component = EntityGetComponent(entity_id, component)[1]
    ComponentSetValue2(component, parameter, value)
end

function SetEntityBlood(entity_id, material) --Blood blood blood
    local damage_models = EntityGetComponent(entity_id, "DamageModelComponent")

    if(damage_models ~= nil) then
        for _, damagemodel in ipairs(damage_models) do
            ComponentSetValue( damagemodel, "blood_material", material )
            ComponentSetValue( damagemodel, "blood_spray_material", material )
        end
    end
end

--Stuff i copied :p

function table.clone(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in pairs(orig) do
            copy[orig_key] = orig_value
        end
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

function TableConcat(t1,t2)
    for i=1,#t2 do
        t1[#t1+1] = t2[i]
    end
    return t1
end

function starts_with(str, start)
    return str:sub(1, #start) == start
 end


